@@ -10,19 +10,53 @@
 
 #include "capplet-util.h"
 
+enum {
+	COL_NAME,
+	COL_ID,
+	N_COLUMNS
+};
+
 #define REVERT_COUNT 20
 
+struct {
+	Rotation  rotation;
+	gchar const    * name;
+} const rotations[] = {
+	{RR_Rotate_0,   N_("normal")},
+	{RR_Rotate_90,  N_("left")},
+	{RR_Rotate_180, N_("inverted")},
+	{RR_Rotate_270, N_("right")}
+};
+
+static Rotation
+display_rotation_from_text(gchar const* text) {
+	int i = 0;
+	g_return_val_if_fail(text, RR_Rotate_0);
+
+	for(; i < G_N_ELEMENTS(rotations); i++) {
+		if(!strcmp(text, _(rotations[i].name))) {
+			break;
+		}
+	}
+
+	g_return_val_if_fail(i < G_N_ELEMENTS(rotations), RR_Rotate_0);
+
+	return rotations[i].rotation;
+}
+
 struct ScreenInfo
 {
   int current_width;
   int current_height;
   SizeID current_size;
   short current_rate;
+  
   Rotation current_rotation;
+  Rotation old_rotation;
+  Rotation rotations;
 
   SizeID old_size;
   short old_rate;
-  Rotation old_rotation;
   
   XRRScreenConfiguration *config;
   XRRScreenSize *sizes;
@@ -30,6 +64,7 @@ struct ScreenInfo
   
   GtkWidget *resolution_widget;
   GtkWidget *rate_widget;
+  GtkWidget *rotate_widget;
 };
 
 struct DisplayInfo {
@@ -44,7 +79,7 @@ struct DisplayInfo {
 static void generate_rate_menu (struct ScreenInfo *screen_info);
 static void generate_resolution_menu(struct ScreenInfo* screen_info);
 
-struct DisplayInfo *
+static struct DisplayInfo *
 read_display_info (GdkDisplay *display)
 {
   struct DisplayInfo *info;
@@ -71,13 +106,14 @@ read_display_info (GdkDisplay *display)
       
       screen_info->current_rate = XRRConfigCurrentRate (screen_info->config);
       screen_info->current_size = XRRConfigCurrentConfiguration (screen_info->config, &screen_info->current_rotation);
-      screen_info->sizes = XRRConfigSizes (screen_info->config, &screen_info->n_sizes);
+      screen_info->sizes        = XRRConfigSizes (screen_info->config, &screen_info->n_sizes);
+      screen_info->rotations    = XRRConfigRotations (screen_info->config, &screen_info->current_rotation);
     }
 
   return info;
 }
 
-void
+static void
 update_display_info (struct DisplayInfo *info, GdkDisplay *display)
 {
   struct ScreenInfo *screen_info;
@@ -107,46 +143,52 @@ update_display_info (struct DisplayInfo *info, GdkDisplay *display)
 
       screen_info->current_rate = XRRConfigCurrentRate (screen_info->config);
       screen_info->current_size = XRRConfigCurrentConfiguration (screen_info->config, &screen_info->current_rotation);
-      screen_info->sizes = XRRConfigSizes (screen_info->config, &screen_info->n_sizes);
+      screen_info->sizes        = XRRConfigSizes (screen_info->config, &screen_info->n_sizes);
+      screen_info->rotations    = XRRConfigRotations (screen_info->config, &screen_info->current_rotation);
     }
 }
 
 static int
 get_current_resolution (struct ScreenInfo *screen_info)
 {
-  GtkWidget *menu;
-  GList *children;
-  GList *child;
-  int i;
+  GtkComboBox* combo = GTK_COMBO_BOX (screen_info->resolution_widget);
+  GtkTreeIter iter;
+  int i = 0;
 
-  i = gtk_option_menu_get_history (GTK_OPTION_MENU (screen_info->resolution_widget));
-  menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (screen_info->resolution_widget));
-  children = gtk_container_get_children (GTK_CONTAINER (menu));
-  child = g_list_nth (children, i);
-
-  if (child != NULL)
-    return GPOINTER_TO_INT (g_object_get_data (child->data, "screen_nr"));
-  else
-    return 0;
+  gtk_combo_box_get_active_iter (combo, &iter);
+  gtk_tree_model_get (gtk_combo_box_get_model (combo), &iter,
+		      COL_ID, &i,
+		      -1);
+  return i;
 }
 
 static int
 get_current_rate (struct ScreenInfo *screen_info)
 {
-  GtkWidget *menu;
-  GList *children;
-  GList *child;
-  int i;
+  GtkComboBox* combo;
+  GtkTreeIter  iter;
+  int i = 0;
 
-  i = gtk_option_menu_get_history (GTK_OPTION_MENU (screen_info->rate_widget));
-  menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (screen_info->rate_widget));
-  children = gtk_container_get_children (GTK_CONTAINER (menu));
-  child = g_list_nth (children, i);
+  combo = GTK_COMBO_BOX (screen_info->rate_widget);
 
-  if (child != NULL)
-    return GPOINTER_TO_INT (g_object_get_data (child->data, "rate"));
-  else
-    return 0;
+  if (gtk_combo_box_get_active_iter (combo, &iter))
+  {
+    gtk_tree_model_get (gtk_combo_box_get_model (combo), &iter,
+  		        COL_ID, &i,
+  		        -1);
+  }
+
+  return i;
+}
+
+static Rotation
+get_current_rotation(struct ScreenInfo* screen_info) {
+	gchar* text;
+	Rotation rot;
+	text = gtk_combo_box_get_active_text (GTK_COMBO_BOX (screen_info->rotate_widget));
+	rot = display_rotation_from_text (text);
+	g_free (text);
+	return rot;
 }
 
 static gboolean
@@ -168,22 +210,25 @@ apply_config (struct DisplayInfo *info)
       Status status;
       GdkWindow *root_window;
       int new_res, new_rate;
+      Rotation new_rot;
 
       screen = gdk_display_get_screen (display, i);
       root_window = gdk_screen_get_root_window (screen);
 
-      new_res = get_current_resolution (screen_info);
+      new_res  = get_current_resolution (screen_info);
       new_rate = get_current_rate (screen_info);
+      new_rot  = get_current_rotation (screen_info);
       
-      if (new_res != screen_info->current_size ||
-	  new_rate != screen_info->current_rate)
+      if (new_res  != screen_info->current_size ||
+	  new_rate != screen_info->current_rate ||
+	  new_rot  != screen_info->current_rotation)
 	{
 	  changed = TRUE; 
 	  status = XRRSetScreenConfigAndRate (xdisplay, 
 					      screen_info->config,
 					      gdk_x11_drawable_get_xid (GDK_DRAWABLE (root_window)),
 					      new_res,
-					      screen_info->current_rotation,
+					      new_rot,
 					      new_rate,
 					      GDK_CURRENT_TIME);
 	}
@@ -324,17 +369,16 @@ resolution_changed_callback (GtkWidget *optionmenu, struct ScreenInfo *screen_in
 static void
 generate_rate_menu (struct ScreenInfo *screen_info)
 {
-  GtkWidget *menu;
-  GtkWidget *menuitem;
+  GtkListStore *store;
+  GtkTreeIter  iter;
   short *rates;
   int nrates, i;
   int size_nr;
   char *str;
   int closest_rate_nr;
   
-  gtk_option_menu_remove_menu (GTK_OPTION_MENU (screen_info->rate_widget));
-
-  menu = gtk_menu_new ();
+  store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);
+  gtk_combo_box_set_model (GTK_COMBO_BOX (screen_info->rate_widget), GTK_TREE_MODEL (store));
 
   size_nr = get_current_resolution (screen_info);
   
@@ -344,40 +388,45 @@ generate_rate_menu (struct ScreenInfo *screen_info)
     {
       str = g_strdup_printf (_("%d Hz"), rates[i]);
 
+      gtk_list_store_append (store, &iter);
+      gtk_list_store_set (store, &iter,
+		          COL_NAME, str,
+			  COL_ID, (int)rates[i],
+			  -1);
+
       if ((closest_rate_nr < 0) ||
 	  (ABS (rates[i] - screen_info->current_rate) <
 	   ABS (rates[closest_rate_nr] - screen_info->current_rate)))
+      {
+	gtk_combo_box_set_active_iter (GTK_COMBO_BOX (screen_info->rate_widget), &iter);
 	closest_rate_nr = i;
-      
-      menuitem = gtk_menu_item_new_with_label (str);
+      }
 
-      g_object_set_data (G_OBJECT (menuitem), "rate", GINT_TO_POINTER ((int)rates[i]));
-	  
       g_free (str);
-      gtk_widget_show (menuitem);
-      gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
     }
 
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (screen_info->rate_widget), menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (screen_info->rate_widget),
-			       closest_rate_nr);
+  g_object_unref (store);
 }
 
 static void
 generate_resolution_menu(struct ScreenInfo* screen_info)
 {
-  GtkWidget *menu;
-  GtkWidget *menuitem;
+  GtkComboBox *combo;
+  GtkListStore* store;
+  GtkTreeIter iter;
   int i, item, current_item;
   XRRScreenSize *sizes;
   char *str;
   SizeID current_size;
   Rotation rot;
 
-  gtk_option_menu_remove_menu(GTK_OPTION_MENU(screen_info->resolution_widget));
-  menu = gtk_menu_new ();
+  combo = GTK_COMBO_BOX (screen_info->resolution_widget);
+  store = gtk_list_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);
+
   current_size = XRRConfigCurrentConfiguration (screen_info->config, &rot);
-  
+
+  gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
+
   current_item = 0;
   item = 0;
   sizes = screen_info->sizes;
@@ -386,49 +435,87 @@ generate_resolution_menu(struct ScreenInfo* screen_info)
       if (i == current_size || show_resolution (sizes[i].width, sizes[i].height))
 	{
 	  str = g_strdup_printf ("%dx%d", sizes[i].width, sizes[i].height);
-	  
+
 	  if (i == current_size)
 	    current_item = item;
-	  
-	  menuitem = gtk_menu_item_new_with_label (str);
 
-	  g_object_set_data (G_OBJECT (menuitem), "screen_nr", GINT_TO_POINTER (i));
-	  
+	  gtk_list_store_append(store, &iter);
+	  gtk_list_store_set(store, &iter,
+			     COL_NAME, str,
+			     COL_ID, i,
+			     -1);
+
 	  g_free (str);
-	  gtk_widget_show (menuitem);
-	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	  item++;
 	}
     }
   
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (screen_info->resolution_widget), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (screen_info->resolution_widget), current_item);
+  gtk_combo_box_set_active (combo, current_item);
 
 	g_signal_connect (screen_info->resolution_widget, "changed", G_CALLBACK (resolution_changed_callback), screen_info);
   
 	gtk_widget_show (screen_info->resolution_widget);
+  g_object_unref (store);
+}
+
+static void
+initialize_combo_layout (GtkCellLayout *layout) {
+  GtkCellRenderer *cell = gtk_cell_renderer_text_new();
+  gtk_cell_layout_pack_start (layout, cell, TRUE);
+  gtk_cell_layout_add_attribute (layout, cell, "text", COL_NAME);
 }
 
 static GtkWidget *
-create_resolution_menu (struct ScreenInfo *screen_info) 
+create_resolution_menu (struct ScreenInfo *screen_info)
 {
-  screen_info->resolution_widget = gtk_option_menu_new ();
+  screen_info->resolution_widget = gtk_combo_box_new ();
   generate_resolution_menu (screen_info);
 
+  initialize_combo_layout (GTK_CELL_LAYOUT (screen_info->resolution_widget));
   return screen_info->resolution_widget;
 }
 
 static GtkWidget *
 create_rate_menu (struct ScreenInfo *screen_info)
 {
-  GtkWidget *optionmenu;
-
-  screen_info->rate_widget = optionmenu = gtk_option_menu_new ();
+  screen_info->rate_widget = gtk_combo_box_new ();
 
   generate_rate_menu (screen_info);
-  
-  gtk_widget_show (optionmenu);
-  return optionmenu;
+
+  initialize_combo_layout (GTK_CELL_LAYOUT (screen_info->rate_widget));
+  gtk_widget_show (screen_info->rate_widget);
+  return screen_info->rate_widget;
+}
+
+static GtkWidget *
+create_rotate_menu (struct ScreenInfo *screen_info)
+{
+  GtkComboBox* combo = NULL;
+  int i, item = 0, current_item = -1;
+
+  screen_info->rotate_widget = gtk_combo_box_new_text ();
+  combo = GTK_COMBO_BOX(screen_info->rotate_widget);
+
+  for (i = 0; i < G_N_ELEMENTS (rotations); i++)
+  {
+    if ((screen_info->rotations & rotations[i].rotation) != 0)
+    {
+      gtk_combo_box_append_text (combo, _(rotations[i].name));
+      if (screen_info->current_rotation == rotations[i].rotation) {
+	current_item = item;
+      }
+      item++;
+    }
+  }
+
+  /* it makes no sense to support only one selection element */
+  gtk_widget_set_sensitive (screen_info->rotate_widget,
+		  gtk_tree_model_iter_n_children (gtk_combo_box_get_model (combo), NULL) > 1);
+
+  gtk_combo_box_set_active (combo, current_item);
+
+  gtk_widget_show (screen_info->rotate_widget);
+  return screen_info->rotate_widget;
 }
 
 static GtkWidget *
@@ -484,6 +571,25 @@ create_screen_widgets (struct ScreenInfo *screen_info, int nr, gboolean no_heade
 		    GTK_FILL | GTK_EXPAND, 0,
 		    0, 0);
   
+  label = gtk_label_new_with_mnemonic (_("R_otation:"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_widget_show (label);
+  gtk_table_attach (GTK_TABLE (table),
+		    label,
+		    0, 1,
+		    2, 3,
+		    GTK_FILL, 0,
+		    0, 0);
+  
+  option_menu = create_rotate_menu (screen_info);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), option_menu);
+  gtk_table_attach (GTK_TABLE (table),
+		    option_menu,
+		    1, 2,
+		    2, 3,
+		    GTK_FILL | GTK_EXPAND, 0,
+		    0, 0);
+  
   if (nr == 0)
     str = g_strdup (_("Default Settings"));
   else
@@ -586,13 +692,13 @@ struct TimeoutData {
   gboolean timed_out;
 };
 
-char *
+static char *
 timeout_string (int time)
 {
   return g_strdup_printf (ngettext ("Testing the new settings. If you don't respond in %d second the previous settings will be restored.", "Testing the new settings. If you don't respond in %d seconds the previous settings will be restored.", time), time);
 }
 
-gboolean
+static gboolean
 save_timeout_callback (gpointer _data)
 {
   struct TimeoutData *data = _data;
@@ -807,7 +913,6 @@ main (int argc, char *argv[])
 		      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 		      NULL);
 
-
   display = gdk_display_get_default ();
   xdisplay = gdk_x11_display_get_xdisplay (display);
   
