@@ -1151,14 +1151,6 @@ setup_meta_tree_view (GtkTreeView *tree_view,
 }
 
 
-static void
-gnome_meta_theme_installer_run_cb (GtkWidget *button,
-				   GtkWidget *parent_window)
-{
-  gnome_theme_installer_run (parent_window, NULL);
-}
-
-
 static void
 gnome_theme_save_clicked (GtkWidget *button,
 			  gpointer   data)
@@ -1246,7 +1238,22 @@ revert_theme_clicked (GtkWidget *button,
   gtk_widget_set_sensitive(WID("meta_theme_revert_button"), FALSE);  
   reverted = TRUE;
 }
-	
+
+
+/* Find out if the lockdown key has been set. Currently returns false on error... */
+static gboolean
+get_lockdown_status ()
+{
+  GConfClient *client;
+  gboolean result;
+
+  client = gconf_client_get_default ();
+  result = gconf_client_get_bool (client, LOCKDOWN_KEY, NULL);
+  g_object_unref (client);
+
+  return result;
+}
+
 static void
 setup_dialog (GladeXML *dialog)
 {
@@ -1277,7 +1284,7 @@ setup_dialog (GladeXML *dialog)
 
   gtk_widget_set_sensitive(WID("meta_theme_revert_button"),FALSE);
 
-  g_signal_connect (G_OBJECT (WID ("meta_theme_install_button")), "clicked", G_CALLBACK (gnome_meta_theme_installer_run_cb), parent);
+  g_signal_connect (G_OBJECT (WID ("meta_theme_install_button")), "clicked", G_CALLBACK (gnome_theme_installer_run_cb), parent);
 
   g_signal_connect (G_OBJECT (WID ("meta_theme_details_button")), "clicked", gnome_theme_details_show, NULL);
 
@@ -1349,6 +1356,16 @@ setup_dialog (GladeXML *dialog)
 
   update_font_button_state (dialog);
   update_background_button_state (dialog);
+
+  if (get_lockdown_status ())
+  {
+	/* theme changing has been disabled by the system administrator */
+	gtk_widget_set_sensitive (WID ("meta_theme_hbox"),FALSE);
+	gtk_widget_set_sensitive (WID ("meta_theme_notebook"),FALSE);
+	gtk_widget_show (WID ("lockdown_hbox"));
+  }
+
+
   gtk_widget_show (parent);
 
 }
@@ -1520,7 +1537,7 @@ gnome_theme_manager_drag_data_received_cb (GtkWidget *widget, GdkDragContext *co
 		gnome_vfs_uri_list_unref (uris);
 	}
 
-	gnome_theme_installer_run (widget, filename);
+	gnome_theme_install_from_uri (filename);
 	g_free (filename);
 }
 
